// <copyright file="ArqCs.ProcessObserver.cs" company="dev.arqendra.net">
// v1.5
// </copyright>
// <author>TrV</author>
// <email>dev(at)arqendra(dot)net</email>
// <date>2010-12</date>
// <summary>Class ProcessObserver.</summary>

using System;

namespace ArqCs
{
    #region constants
    /// <summary>
    /// Process kind.
    /// </summary>
    public enum ProcessKind
    {
        /// <summary>
        /// Process kind is undefined.
        /// </summary>
        Undefined,

        /// <summary>
        /// Process is an executable.
        /// </summary>
        Executable,

        /// <summary>
        /// Process is a service.
        /// </summary>
        Service
    }

    /// <summary>
    /// Process status.
    /// </summary>
    public enum ProcessStatus
    {
        /// <summary>
        /// Process status is unknown.
        /// </summary>
        Unknown,

        /// <summary>
        /// Process is stopped.
        /// </summary>
        Stopped,

        /// <summary>
        /// Process is started.
        /// </summary>
        Started
    }
    #endregion

    #region ProcessStatusEventArgs
    /// <summary>
    /// Holds information about process status.
    /// </summary>
    public sealed class ProcessStatusEventArgs : EventArgs
    {
        #region attributes
        /// <summary>
        /// Thread identificator.
        /// </summary>
        private string id;

        /// <summary>
        /// Process name.
        /// </summary>
        private string name;

        /// <summary>
        /// Process status.
        /// </summary>
        private ProcessStatus status;
        #endregion

        #region properties
        /// <summary>
        /// Gets identificator.
        /// </summary>
        /// <value>Identificator string.</value>
        public string Id
        {
            get { return this.id; }
        }

        /// <summary>
        /// Gets process name.
        /// </summary>
        /// <value>Executable file name or service name.</value>
        public string Name
        {
            get { return this.name; }
        }

        /// <summary>
        /// Gets process status.
        /// </summary>
        /// <value>Process status.</value>
        public ProcessStatus Status
        {
            get { return this.status; }
        }
        #endregion

        #region constructor
        /// <summary>
        /// Initializes a new instance of the ProcessStatusEventArgs class.
        /// </summary>
        /// <param name="id">Thread identificator.</param>
        /// <param name="name">Process name.</param>
        /// <param name="status">Process status.</param>
        public ProcessStatusEventArgs(string id, string name, ProcessStatus status)
        {
            this.id = id;
            this.name = name;
            this.status = status;
        }
        #endregion
    }
    #endregion

    #region ProcessObserver
    /// <summary>
    /// Observes process.
    /// </summary>
    public abstract class ProcessObserver : LoopThread
    {
        #region attributes
        /// <summary>
        /// Process kind.
        /// </summary>
        private ProcessKind kind;

        /// <summary>
        /// Process name (executable file name or service name).
        /// </summary>
        private string name;

        /// <summary>
        /// Observed process status.
        /// </summary>
        private ProcessStatus status;
        #endregion

        #region properties
        /// <summary>
        /// Gets or sets observed process kind.
        /// </summary>
        /// <value>Process kind (Undefined/Executable/Service).</value>
        /// <exception cref="InvalidOperationException">Observed process kind has been modified while thread is running.</exception>
        public ProcessKind Kind
        {
            get {
                return this.kind;
            }

            set {
                if ( !this.Running ) {
                    this.kind = value;
                }
                else {
                    throw new System.InvalidOperationException("Observed process kind cannot be modified while thread is running");
                }
            }
        }

        /// <summary>
        /// Gets or sets observed process name.
        /// </summary>
        /// <value>Executable file name or service name.</value>
        /// <exception cref="InvalidOperationException">Observed process name has been modified while thread is running.</exception>
        public string Name
        {
            get {
                return this.name;
            }

            set {
                if ( !this.Running ) {
                    this.name = value;
                }
                else {
                    throw new System.InvalidOperationException("Observed process name cannot be modified while thread is running");
                }
            }
        }

        /// <summary>
        /// Gets or sets scan frequency.
        /// </summary>
        /// <value>Scan frequency [seconds].</value>
        /// <exception cref="InvalidOperationException">Scan frequency has been modified while thread is running.</exception>
        public int ScanFrequency
        {
            get { return this.LoopTime; }
            set { this.LoopTime = value; }
        }

        /// <summary>
        /// Gets process status.
        /// </summary>
        /// <value>Process status (Unknown/Started/Stopped).</value>
        public ProcessStatus Status
        {
            get { return this.status; }
        }

        /// <summary>
        /// Gets a value indicating whether observer is running or not.
        /// </summary>
        /// <value>True/False if observer is/is not running.</value>
        public new bool Running
        {
            // This property has been overriden just to update documentation
            get { return base.Running; }
        }

        /// <summary>
        /// Gets a value indicating whether observer is suspended or not.
        /// </summary>
        /// <value>True/False if observer is/is not suspended.</value>
        public new bool Suspended
        {
            // This property has been overriden just to update documentation
            get { return base.Suspended; }
        }
        #endregion

        #region constructor
        /// <summary>
        /// Initializes a new instance of the ProcessObserver class.
        /// </summary>
        /// <param name="id">Thread identificator.</param>
        /// <param name="name">Observed process name.</param>
        /// <param name="scanFrequency">Scan frequency [seconds].</param>
        protected ProcessObserver(string id, string name, int scanFrequency)
            : base(500, scanFrequency * 1000) // polls thread every 0.5 second, sleeps for 10 seconds
        {
            this.Id = id;

            this.kind = ProcessKind.Undefined;
            this.name = name;

            this.status = ProcessStatus.Unknown; // initializes observed process status
        }
        #endregion

        #region events
        /// <summary>
        /// Occurs when process status has changed.
        /// </summary>
        public event EventHandler<ProcessStatusEventArgs> StatusChanged;
        #endregion

        #region management methods
        /// <summary>
        /// Launches process observation.
        /// </summary>
        public new void Run()
        {
            base.Run();
        }

        /// <summary>
        /// Forces observation to execute as soon as possible (on next thread poll).
        /// </summary>
        public new void ForceLoopNow()
        {
            base.ForceLoopNow();
        }

        /// <summary>
        /// Asks the observer to terminate gracefully as soon as possible (on next thread poll).
        /// </summary>
        public new void Stop()
        {
            base.Stop();
        }

        /// <summary>
        /// Asks the observer to suspend as soon as possible (on next thread poll).
        /// </summary>
        public new void Suspend()
        {
            base.Suspend();
        }

        /// <summary>
        /// Asks the observer to resume as soon as possible (on next thread poll).
        /// </summary>
        public new void Resume()
        {
            base.Resume();
        }
        #endregion

        #region overriden methods
        /// <summary>
        /// Tests process status.
        /// </summary>
        protected sealed override void DoLoop()
        {
            ProcessStatus currentStatus = this.TestStatus(); // gets current process status

            if ( this.status != currentStatus ) { // if observed process status has changed since last poll
                this.status = currentStatus;
                if ( this.StatusChanged != null ) { // if any event handler is attached to the event
                    this.StatusChanged(this, new ProcessStatusEventArgs(this.Id, this.name, this.status)); // sends event
                }
            }
        }

        /// <summary>
        /// Nothing to do.
        /// </summary>
        protected sealed override void DoFirst()
        {
            this.status = ProcessStatus.Unknown; // re-initializes observed process status
        }

        /// <summary>
        /// Nothing to do.
        /// </summary>
        protected sealed override void DoLast()
        {
        }

        /// <summary>
        /// Stands for current object.
        /// </summary>
        /// <returns>Information about current object.</returns>
        public override string ToString()
        {
            return base.ToString() + " - " + this.name;
        }
        #endregion

        #region specific methods
        /// <summary>
        /// Tests process status.
        /// </summary>
        /// <returns>Current process status.</returns>
        protected abstract ProcessStatus TestStatus();
        #endregion
    }
    #endregion
}
