// <copyright file="ArqCs.ServiceObserver.cs" company="dev.arqendra.net">
// v1.5
// </copyright>
// <author>TrV</author>
// <email>dev(at)arqendra(dot)net</email>
// <date>2010-12</date>
// <summary>Class ServiceObserver.</summary>

using System;

namespace ArqCs
{
    #region ServiceObserver
    /// <summary>
    /// Observes service.
    /// </summary>
    public class ServiceObserver : ProcessObserver, IDisposable
    {
        #region attributes
        /// <summary>
        /// ServiceController object.
        /// </summary>
        private System.ServiceProcess.ServiceController serviceController;
        #endregion

        #region constructors & destructors
        /// <summary>
        /// Initializes a new instance of the ServiceObserver class.
        /// </summary>
        /// <param name="serviceName">Observed service name.</param>
        /// <param name="scanFrequency">Scan frequency [seconds].</param>
        public ServiceObserver(string serviceName, int scanFrequency)
            : base(String.Empty, serviceName, scanFrequency)
        {
            this.Kind = ProcessKind.Service;

            this.serviceController = new System.ServiceProcess.ServiceController(serviceName); // initializes observed service controller
        }

        /// <summary>
        /// Initializes a new instance of the ServiceObserver class.
        /// </summary>
        /// <param name="id">Thread identificator.</param>
        /// <param name="serviceName">Observed service name.</param>
        /// <param name="scanFrequency">Scan frequency [seconds].</param>
        public ServiceObserver(string id, string serviceName, int scanFrequency)
            : base(id, serviceName, scanFrequency)
        {
            this.Kind = ProcessKind.Service;

            this.serviceController = new System.ServiceProcess.ServiceController(serviceName); // initializes observed service controller
        }

        /// <summary>
        /// Cleans resources.
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Cleans resources.
        /// </summary>
        /// <param name="disposing">True if managed resources should be disposed; otherwise, false.</param>
        protected virtual void Dispose(bool disposing)
        {
            if ( disposing ) {
                this.serviceController.Dispose();
            }
        }
        #endregion

        #region overriden methods
        /// <summary>
        /// Tests service status.
        /// </summary>
        /// <returns>Current service status.</returns>
        protected sealed override ProcessStatus TestStatus()
        {
            ProcessStatus currentStatus = ProcessStatus.Unknown; // current process status

            this.serviceController.ServiceName = this.Name;
            this.serviceController.Refresh(); // refreshes service data

            try {
                if ( this.serviceController.Status == System.ServiceProcess.ServiceControllerStatus.Running ) { // if service is running
                    currentStatus = ProcessStatus.Started; // marks it as 'started'
                }
                else { // if service is not running
                    currentStatus = ProcessStatus.Stopped; // marks it as 'stopped'
                }
            }
            catch ( System.InvalidOperationException ) { } // (System.ServiceProcess.ServiceController)
            catch ( System.ComponentModel.Win32Exception ) { } // (System.ServiceProcess.ServiceController.Status)

            return currentStatus;
        }
        #endregion
    }
    #endregion
}
